package com.landray.kmss.sys.attachment.service.spring;

import com.landray.kmss.sys.filestore.model.SysAttCatalog;
import com.landray.kmss.sys.attachment.model.SysAttMain;
import org.springframework.transaction.TransactionStatus;
import java.io.IOException;
import com.landray.kmss.util.TransactionUtils;
import java.util.Map;
import java.net.URLEncoder;
import java.io.InputStream;
import com.landray.kmss.sys.attachment.io.DecryptionInputStream;
import java.io.FileInputStream;
import java.io.OutputStream;
import com.landray.kmss.sys.attachment.io.IOUtil;
import java.io.FileOutputStream;
import com.landray.kmss.util.IDGenerator;
import java.io.File;
import com.landray.kmss.util.FileUtil;
import java.util.HashMap;
import com.sunbor.web.tag.Page;
import java.util.Iterator;
import com.landray.kmss.sys.filestore.location.interfaces.ISysFileLocationProxyService;
import com.landray.kmss.common.dao.IBaseDao;
import java.util.List;
import com.landray.kmss.util.ArrayUtil;
import java.text.ParseException;
import com.landray.kmss.util.StringUtil;
import com.landray.kmss.sys.filestore.model.SysAttFile;
import com.landray.kmss.common.dao.HQLInfo;
import com.landray.kmss.sys.filestore.location.util.SysFileLocationUtil;
import com.alibaba.fastjson.JSON;
import com.landray.kmss.sys.quartz.interfaces.SysQuartzJobContext;
import java.util.Date;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import com.landray.kmss.sys.quartz.interfaces.ISysQuartzModel;
import com.landray.kmss.sys.quartz.interfaces.ISysQuartzModelContext;
import com.landray.kmss.sys.quartz.interfaces.SysQuartzModelContext;
import com.landray.kmss.common.model.IBaseModel;
import com.landray.kmss.util.ResourceUtil;
import org.json.simple.JSONObject;
import com.landray.kmss.util.SpringBeanUtil;
import org.slf4j.LoggerFactory;
import com.landray.kmss.sys.filestore.dao.ISysAttUploadDao;
import com.landray.kmss.sys.quartz.interfaces.ISysQuartzCoreService;
import com.landray.kmss.sys.cache.KmssCache;
import org.slf4j.Logger;
import com.landray.kmss.sys.attachment.service.ISysAttToggleService;
import com.landray.kmss.common.service.BaseServiceImp;

public class SysAttToggleServiceImp extends BaseServiceImp implements ISysAttToggleService
{
    private static final Logger logger;
    private static KmssCache attachmentTogglingCache;
    private static final String ATT_TOGGLING_KEY = "SysAttMainCoreInnerService.attachmentTogglingCache";
    private static final String ATT_TOGGLING_MODEL_ID = "32f512618f7d96315b932ed6f012b18f";
    private ISysQuartzCoreService sysQuartzCoreService;
    private ISysAttUploadDao sysAttUploadDao;
    
    static {
        logger = LoggerFactory.getLogger(SysAttToggleServiceImp.class);
        SysAttToggleServiceImp.attachmentTogglingCache = new KmssCache((Class)SysAttToggleServiceImp.class);
    }
    
    public void setSysQuartzCoreService(final ISysQuartzCoreService sysQuartzCoreService) {
        this.sysQuartzCoreService = sysQuartzCoreService;
    }
    
    public ISysAttUploadDao getSysAttUploadDao() {
        if (this.sysAttUploadDao == null) {
            this.sysAttUploadDao = (ISysAttUploadDao)SpringBeanUtil.getBean("sysAttUploadDao");
        }
        return this.sysAttUploadDao;
    }
    
    private boolean isAttachmentToggling() {
        final Object isAttachmentToggling = SysAttToggleServiceImp.attachmentTogglingCache.get("SysAttMainCoreInnerService.attachmentTogglingCache");
        return isAttachmentToggling != null && isAttachmentToggling instanceof Boolean && (boolean)isAttachmentToggling;
    }
    
    private void setAttachmentToggling(final boolean isAttachmentToggling) {
        SysAttToggleServiceImp.attachmentTogglingCache.put("SysAttMainCoreInnerService.attachmentTogglingCache", (Object)isAttachmentToggling);
    }
    
    public JSONObject saveToggleAttchment(final String operate, final String source, final String target, final String startDate, final String endDate, final int toggleStartTime, final int toggleEndTime) throws Exception {
        final JSONObject jsonObj = new JSONObject();
        if ("status".equals(operate)) {
            jsonObj.put((Object)"toggling", (Object)this.isAttachmentToggling());
        }
        else if ("start".equals(operate)) {
            jsonObj.put((Object)"toggling", (Object)this.isAttachmentToggling());
            final String time = this.job(source, target, startDate, endDate, toggleStartTime, toggleEndTime);
            jsonObj.put((Object)"toggleMsg", (Object)String.format(ResourceUtil.getString("sys-attachment:attachment.filestore.toggle.toggleMsg"), time));
        }
        else if ("end".equals(operate)) {
            this.setAttachmentToggling(false);
            jsonObj.put((Object)"toggling", (Object)false);
            final SysAttToggle attToggle = new SysAttToggle((SysAttToggle)null);
            attToggle.setFdId("32f512618f7d96315b932ed6f012b18f");
            this.sysQuartzCoreService.delete((IBaseModel)attToggle, "SysAttMainCoreInnerService.attachmentTogglingCache");
        }
        return jsonObj;
    }
    
    private String job(final String source, final String target, final String startDate, final String endDate, final int toggleStartTime, final int toggleEndTime) throws Exception {
        final JSONObject obj = new JSONObject();
        obj.put((Object)"source", (Object)source);
        obj.put((Object)"target", (Object)target);
        obj.put((Object)"startDate", (Object)startDate);
        obj.put((Object)"endDate", (Object)endDate);
        obj.put((Object)"toggleStartTime", (Object)toggleStartTime);
        obj.put((Object)"toggleEndTime", (Object)toggleEndTime);
        final SysAttToggle attToggle = new SysAttToggle((SysAttToggle)null);
        attToggle.setFdId("32f512618f7d96315b932ed6f012b18f");
        final SysQuartzModelContext sysQuartzModelContext = new SysQuartzModelContext();
        final String cron = "0 0 " + toggleStartTime + " * * ?";
        sysQuartzModelContext.setQuartzCronExpression(cron);
        sysQuartzModelContext.setQuartzJobMethod("saveAndDoToggleAttchment");
        sysQuartzModelContext.setQuartzKey("SysAttMainCoreInnerService.attachmentTogglingCache");
        sysQuartzModelContext.setQuartzParameter(obj.toJSONString());
        sysQuartzModelContext.setQuartzSubject(ResourceUtil.getString("sys-attachment:attachment.filestore.toggle.task"));
        sysQuartzModelContext.setQuartzJobService("sysAttToggleService");
        sysQuartzModelContext.setQuartzLink("");
        this.sysQuartzCoreService.saveScheduler((ISysQuartzModelContext)sysQuartzModelContext, (ISysQuartzModel)attToggle);
        final Calendar c = Calendar.getInstance();
        c.setTimeInMillis(System.currentTimeMillis());
        c.set(11, toggleStartTime);
        c.set(12, 0);
        c.set(13, 0);
        Date executeTime = null;
        if (c.getTimeInMillis() > System.currentTimeMillis()) {
            executeTime = c.getTime();
        }
        else {
            c.add(5, 1);
            executeTime = c.getTime();
        }
        return new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(executeTime);
    }
    
    public void saveAndDoToggleAttchment(final SysQuartzJobContext jobContext) {
        this.setAttachmentToggling(true);
        final IBaseDao baseDao = this.getBaseDao();
        final String param = jobContext.getParameter();
        final com.alibaba.fastjson.JSONObject json = JSON.parseObject(param);
        final String source = json.getString("source");
        final String target = json.getString("target");
        final String startDate = json.getString("startDate");
        final String endDate = json.getString("endDate");
        final String toggleStartTime = json.getString("toggleStartTime");
        final String toggleEndTime = json.getString("toggleEndTime");
        final Calendar c = Calendar.getInstance();
        c.setTimeInMillis(System.currentTimeMillis());
        c.set(10, Integer.valueOf(toggleEndTime));
        c.set(12, 0);
        c.set(13, 0);
        final Long endExecuteTime = c.getTime().getTime();
        final SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd");
        final ISysFileLocationProxyService sourceService = SysFileLocationUtil.getProxyService(source);
        final ISysFileLocationProxyService targetService = SysFileLocationUtil.getProxyService(target);
        final HQLInfo hql = new HQLInfo();
        hql.setModelName(SysAttFile.class.getName());
        hql.setWhereBlock("fdAttLocation=:fdAttLocation");
        try {
            if (StringUtil.isNotNull(startDate)) {
                hql.setWhereBlock(String.valueOf(hql.getWhereBlock()) + " and docCreateTime>:startDate");
                hql.setParameter("startDate", (Object)dateFormat.parse(startDate));
            }
            if (StringUtil.isNotNull(endDate)) {
                hql.setWhereBlock(String.valueOf(hql.getWhereBlock()) + " and docCreateTime<:endDate");
                hql.setParameter("endDate", (Object)dateFormat.parse(endDate));
            }
        }
        catch (final ParseException e) {
            e.printStackTrace();
        }
        hql.setParameter("fdAttLocation", (Object)source);
        hql.setRowSize(1000);
        hql.setPageNo(0);
        List<SysAttFile> attFileList = null;
        Iterator<SysAttFile> iterator = null;
        Page page = null;
        do {
            hql.setPageNo(hql.getPageNo() + 1);
            try {
                page = this.findPage(hql);
                attFileList = page.getList();
                iterator = this.toogleFiles(baseDao, source, target, endExecuteTime, sourceService, targetService, attFileList);
            }
            catch (final Exception e2) {
                e2.printStackTrace();
            }
        } while (page.getTotal() > hql.getPageNo() * hql.getRowSize() && !ArrayUtil.isEmpty((List)attFileList) && this.isAttachmentToggling() && System.currentTimeMillis() < endExecuteTime);
        this.setAttachmentToggling(false);
        try {
            if (iterator != null && iterator.hasNext()) {
                this.job(source, target, startDate, endDate, Integer.valueOf(toggleStartTime), Integer.valueOf(toggleEndTime));
            }
            else {
                final SysQuartzModelContext sysQuartzModelContext = new SysQuartzModelContext();
                sysQuartzModelContext.setQuartzKey("SysAttMainCoreInnerService.attachmentTogglingCache");
                sysQuartzModelContext.setQuartzJobService("sysAttToggleService");
                final SysAttToggle attToggle = new SysAttToggle((SysAttToggle)null);
                attToggle.setFdId("32f512618f7d96315b932ed6f012b18f");
                sysQuartzModelContext.setQuartzEnabled(Boolean.valueOf(false));
                this.sysQuartzCoreService.saveScheduler((ISysQuartzModelContext)sysQuartzModelContext, (ISysQuartzModel)attToggle);
            }
        }
        catch (final NumberFormatException e3) {
            e3.printStackTrace();
        }
        catch (final Exception e4) {
            e4.printStackTrace();
        }
    }
    
    private Iterator<SysAttFile> toogleFiles(final IBaseDao baseDao, final String source, final String target, final Long endExecuteTime, final ISysFileLocationProxyService sourceService, final ISysFileLocationProxyService targetService, final List<SysAttFile> attFileList) {
        final Iterator<SysAttFile> iterator = attFileList.iterator();
        final Map<String, String> header = new HashMap<String, String>();
        while (this.isAttachmentToggling() && System.currentTimeMillis() < endExecuteTime && iterator.hasNext()) {
            final SysAttFile sysAttFile = iterator.next();
            final String pathPrefix = (sysAttFile.getFdCata() == null) ? null : sysAttFile.getFdCata().getFdPath();
            TransactionStatus status = null;
            InputStream in = null;
            try {
                if (!targetService.doesFileExist(sysAttFile.getFdFilePath(), pathPrefix)) {
                    final String tempDir = FileUtil.getSystemTempPath();
                    final String tempFile = String.valueOf(tempDir) + File.separator + sysAttFile.getFdId() + "-" + IDGenerator.generateID() + ".toggle.temp";
                    if ("server".equals(source)) {
                        final InputStream is = sourceService.readFile(sysAttFile.getFdFilePath(), pathPrefix);
                        IOUtil.write(is, (OutputStream)new FileOutputStream(tempFile));
                        in = (InputStream)new DecryptionInputStream((InputStream)new FileInputStream(tempFile), 2);
                    }
                    else {
                        in = sourceService.readFile(sysAttFile.getFdFilePath(), pathPrefix);
                    }
                    if (in == null) {
                        SysAttToggleServiceImp.logger.info("===========SysAttMainCoreInnerService.toggleAttchment faild,source not exists:" + sysAttFile.getFdFilePath() + "(" + source + "->" + target + ")");
                    }
                    if (!"server".equals(target)) {
                        final SysAttMain attMain = this.getAttMainByPath(sysAttFile.getFdId());
                        String fileName = (attMain == null) ? "" : attMain.getFdFileName();
                        fileName = URLEncoder.encode(fileName, "utf-8");
                        header.put("Content-Disposition", "filename=" + fileName);
                    }
                    targetService.writeFile(in, sysAttFile.getFdFilePath(), (Map)header);
                    final File f = new File(tempFile);
                    if (f.exists()) {
                        f.delete();
                    }
                }
                this.toggleRelativeDirAndFiles(sourceService, targetService, sysAttFile.getFdFilePath(), source, pathPrefix, true);
                sysAttFile.setFdAttLocation(target);
                if ("server".equals(target)) {
                    final SysAttCatalog fdCata = this.getSysAttUploadDao().getDefultCatalog();
                    sysAttFile.setFdCata(fdCata);
                }
                status = TransactionUtils.beginNewTransaction();
                baseDao.update((IBaseModel)sysAttFile);
                TransactionUtils.getTransactionManager().commit(status);
                SysAttToggleServiceImp.logger.info("===========SysAttMainCoreInnerService.toggleAttchment success:" + sysAttFile.getFdFilePath() + "(" + source + "->" + target + ")");
            }
            catch (final Exception e) {
                e.printStackTrace();
                if (status != null) {
                    TransactionUtils.getTransactionManager().rollback(status);
                }
                SysAttToggleServiceImp.logger.error("===========SysAttMainCoreInnerService.toggleAttchment error:" + sysAttFile.getFdFilePath() + "(" + source + "->" + target + ")", e);
                if (in != null) {
                    try {
                        in.close();
                    }
                    catch (final IOException e2) {
                        e2.printStackTrace();
                    }
                    continue;
                }
                continue;
            }
            finally {
                if (in != null) {
                    try {
                        in.close();
                    }
                    catch (final IOException e2) {
                        e2.printStackTrace();
                    }
                }
            }
            if (in != null) {
                try {
                    in.close();
                }
                catch (final IOException e2) {
                    e2.printStackTrace();
                }
            }
        }
        return iterator;
    }
    
    private void toggleRelativeDirAndFiles(final ISysFileLocationProxyService sourceService, final ISysFileLocationProxyService targetService, final String filePath, final String source, final String pathPrefix, final boolean validPrefix) throws Exception {
        final String parent = filePath.substring(0, filePath.lastIndexOf("/") + 1);
        final String fileId = filePath.substring(filePath.lastIndexOf("/") + 1);
        final List<String> relativeFileList = sourceService.listFileNames(parent, pathPrefix);
        for (final String path : relativeFileList) {
            final String prefix = path.replace(parent, "");
            if (validPrefix) {
                if (!prefix.startsWith(fileId)) {
                    continue;
                }
                if (prefix.equals(fileId)) {
                    continue;
                }
            }
            InputStream in = null;
            final String tempDir = FileUtil.getSystemTempPath();
            final String tempFile = String.valueOf(tempDir) + File.separator + prefix + "-" + IDGenerator.generateID() + ".toggle.temp";
            Label_0347: {
                try {
                    if ("server".equals(source)) {
                        final InputStream is = sourceService.readFile(path, pathPrefix);
                        IOUtil.write(is, (OutputStream)new FileOutputStream(tempFile));
                        in = (InputStream)new DecryptionInputStream((InputStream)new FileInputStream(tempFile), 2);
                    }
                    else {
                        in = sourceService.readFile(path, pathPrefix);
                    }
                    if (in == null) {}
                    targetService.writeFile(in, path);
                }
                catch (final Exception e) {
                    e.printStackTrace();
                    if (in != null) {
                        try {
                            in.close();
                        }
                        catch (final Exception e2) {
                            e2.printStackTrace();
                        }
                    }
                    break Label_0347;
                }
                finally {
                    if (in != null) {
                        try {
                            in.close();
                        }
                        catch (final Exception e2) {
                            e2.printStackTrace();
                        }
                    }
                }
                if (in != null) {
                    try {
                        in.close();
                    }
                    catch (final Exception e2) {
                        e2.printStackTrace();
                    }
                }
            }
            final File f = new File(tempFile);
            if (f.exists()) {
                f.delete();
            }
        }
        final List<String> relativeSubDirList = sourceService.listSubDirNames(parent, pathPrefix);
        for (String pathDir : relativeSubDirList) {
            final String path2 = pathDir.replace(parent, "");
            if (validPrefix) {
                if (!path2.startsWith(fileId)) {
                    continue;
                }
                if (path2.equals(fileId)) {
                    continue;
                }
            }
            pathDir = (pathDir.endsWith("/") ? pathDir : (String.valueOf(pathDir) + "/"));
            this.toggleRelativeDirAndFiles(sourceService, targetService, pathDir, source, pathPrefix, false);
        }
    }
    
    public SysAttMain getAttMainByPath(final String fileId) throws Exception {
        final HQLInfo hql = new HQLInfo();
        hql.setModelName(SysAttMain.class.getName());
        hql.setWhereBlock("fdFileId=:fdFileId");
        hql.setParameter("fdFileId", (Object)fileId);
        hql.setOrderBy("docCreateTime asc");
        final SysAttMain att = (SysAttMain)this.findFirstOne(hql);
        return att;
    }
    
    private class SysAttToggle extends SysAttMain implements ISysQuartzModel
    {
        private static final long serialVersionUID = -5661731258457612569L;
    }
}
